﻿using System;
using System.Numerics;
using EMS.Math.Cx;


namespace Cx.Net_Demo
{
    class Program
    {
        //User defined function
        static TCALC pow3(TCALC x)
        {
            //Must convert to Complex first
            Complex cmplx = new Complex(x.real, x.imag);

            //Do calculation
            Complex tmp = cmplx * cmplx * cmplx;

            //Transform result to TCalc data type
            TCALC result;
            result.real = tmp.Real;
            result.imag = tmp.Imaginary;
            return result;
        }



        unsafe static void Main(string[] args)
        {
            string mathExp = "x+2.3i*(1-sin(y/3.14)^2)-z";
            string varList = "x,y,z";
            TCALC[] data = new TCALC[3];
            data[0].real = 1.2; data[0].imag = 2.3;
            data[1].real = -3; data[1].imag = 0.005;
            data[2].real = 0.5; data[2].imag = -0.123;
            TCALC res = default(TCALC);
            Complex cmpResult = default(Complex);


            //Initialize CxTranslator algorithm before we can use any function.
            Translator cx = new Translator();

            try
            {
                //****************************************************
                //Evaluate math expression for a given point "data"
                res = cx.Evaluate(mathExp, varList, data);

                data[0].real = 2.3; data[0].imag = 1.3;
                data[1].real = -5; data[1].imag = 0.123;
                res = cx.Evaluate("x/y+5-3.1j+(x^2-y^2)", "x,y", data);

                //****************************************************
                //Add user's function to CxTranslator
                cx.AddFunction("p3", pow3);

                //Calculate a set of points for one math expression
                //Build it first
                cx.Build("3.14*p3(x)+sin(y)-sqrt(z+x)", "x,y,z");


                for (int i = 0; i < 100; i++)
                {
                    data[0].real = data[0].real + 1; data[0].imag = i;
                    data[1].real = i / 200; data[1].imag = 0.005 * i;
                    data[2].real = -5; data[2].imag = data[0].real * 0.123;

                    //Do calculation for a given point
                    res = cx.CalcFor(data);
                    cmpResult = new Complex(cmpResult.Real + res.real, cmpResult.Imaginary + res.imag);
                }

                //****************************************************
                //Memory for data must be pinned in order to use 
                //"Compile" and "Run" functions.
                TCALC* pinData = stackalloc TCALC[3];
                cx.Compile(mathExp, varList, pinData);

                res = default(TCALC);
                for (int i = 0; i < 10000; i++)
                {
                    pinData[0].real = pinData[0].real + 0.0001; pinData[0].imag = i;
                    pinData[1].real = i / 2000; pinData[1].imag = i / 10000;
                    pinData[2].real = -5; pinData[2].imag = pinData[0].real * 0.0123;

                    //Do calculation for a given point
                    res = cx.Run();
                }


                //****************************************************
                //Negative test
                cx.RemoveFunction("p3");
                res = cx.Evaluate("3.14*p3(x)+sin(y)-sqrt(z+x)", "x,y,z", data);


            }
            catch (TranslatorException ex)
            {
                Console.WriteLine(ex.m_errCode);
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.m_errPosition);
            }

        }
    }
}
